/* 
 * color class routines
 */
#include "declare.h"
#include "color.h"

Colors cols;
extern Colormap cmap;

String symbolicName[256];

/*
 * copy the root colormap and allocate every cell that is not black
 */
#ifdef _NO_PROTO
createColormap()
#else
void createColormap(void)
#endif
{
XColor cell, dummycell;
Colormap rootmap;
int i;
Pixel pixels[256], pix;
XColor storeCells[256], queryCells[256];
Pixel freePixels[256];
int numStore, numFree;

    for (i=0; i<256; i++)
    {
	cols.cells[i].red = 0;
	cols.cells[i].green = 0;
	cols.cells[i].blue = 0;
	symbolicName[i] = NULL;
    }
    symbolicName[0] = XtNewString("background");

    rootmap = DefaultColormap(display, screen);

    /* make all free cells be black */
    if (!appData.emptyCmap)
    {
	numStore = 0;
	cell.red = cell.green = cell.blue = 0;
	cell.flags = DoRed | DoGreen | DoBlue;
	while  (XAllocColorCells(display, rootmap, False, NULL, 0, &pix, 1))
	{
	    pixels[numStore] = cell.pixel = pix;
	    storeCells[numStore++] = cell;
	}
	/* now free them up */
	if (numStore)
	{
	    XStoreColors(display, rootmap, storeCells, numStore);
	    XFreeColors(display, rootmap, pixels, numStore, 0);
	}
    }

    cmap = XCreateColormap(display, RootWindow(display, screen),
	DefaultVisual(display, screen), AllocNone);

    for (i=0; i<256; i++)
    {
	cols.changable[i] = False;
	cols.freeCell[i] = False;
    }
    /*
     * make colorcell 0 be black (background) and sharable
     */
    XAllocNamedColor(display, cmap, "black", &cell, &dummycell);
    if (cell.pixel != 0)
    {
	printf("could not set pixel 0 to black!\n");
	exit(-1);
    }
    cols.changable[0] = False;
    cols.freeCell[0] = False;
    if (appData.emptyCmap)
	return;

    /* allocate all cells */
    XAllocColorCells(display, cmap, True, NULL, 0, pixels, 255);

    /* store colors for non-black cells and BlackPixel */
    numStore = numFree = 0;
    for (i=0; i<256; i++)
	queryCells[i].pixel = i;
    XQueryColors(display, rootmap, queryCells, 256);

    for (i=1; i<256; i++)
    {

	cell = queryCells[i];
	cell.red   &= 0xff00;
	cell.green &= 0xff00;
	cell.blue  &= 0xff00;

	/* store new color if any */
	if (cell.red || cell.green || cell.blue || 
		i == BlackPixel(display, screen))
	{
	    storeCells[numStore++] = cell;
	}
	else
	{
	    /* free unused cell */
	    freePixels[numFree++] = i;
	}
    }
    if (numStore)
	XStoreColors(display, cmap, storeCells, numStore);
    if (numFree)
	XFreeColors(display, cmap, freePixels, numFree, 0);
}

#ifdef _NO_PROTO
void getRGB(pixel, red, green, blue, changable)
int pixel; int *red; int *green; int *blue; int *changable;
#else
void getRGB(int pixel, int *red, int *green, int *blue, int *changable)
#endif
{
    *red   = cols.cells[pixel].red;
    *green = cols.cells[pixel].green;
    *blue  = cols.cells[pixel].blue;
    *changable = cols.changable[pixel];
}

#ifdef _NO_PROTO
void setRGB(pixel, red, green, blue)
int pixel; int red; int green; int blue;
#else
void setRGB(int pixel, int red, int green, int blue)
#endif
{
XColor cell;
    if (!cols.changable[pixel])
	return;
    cell.red   = cols.cells[pixel].red  = red;
    cell.green = cols.cells[pixel].green= green;
    cell.blue  = cols.cells[pixel].blue = blue;
    cell.pixel = pixel;
    cell.flags = DoRed | DoGreen | DoBlue;
    XStoreColor(display, cmap, &cell);
}

#ifdef _NO_PROTO
void newColor(pixel, name)
int *pixel; char *name;
#else
void newColor(int *pixel, char *name)
#endif
{
unsigned long newpix;
XColor c;
    if (XAllocColorCells(display, cmap, False, NULL, 0, &newpix, 1))
    {
	if (!XParseColor(display, cmap, name, &c))
	{
	    c.red = c.green = c.blue = 0;
	}
	c.pixel = newpix;
	XStoreColor(display, cmap, &c);
	cols.cells[newpix].red  = c.red & 0xff00;
	cols.cells[newpix].green= c.green & 0xff00;
	cols.cells[newpix].blue = c.blue & 0xff00;
	cols.changable[newpix] = True;
	cols.freeCell[newpix] = False;
	*pixel = (int)newpix;
    }
    else
	*pixel = -1;
}

#ifdef _NO_PROTO
int compressColor(c)
int c;
#else
int compressColor(int c)
#endif
{
int red, green, blue, dummy;
int i;
    getRGB(c, &red, &green, &blue, &dummy);
    for (i=0; i<c; i++)
    {
	if (cols.cells[i].red == red && cols.cells[i].green == green &&
		cols.cells[i].blue == blue)
	    break;
    }
    return(i);
}

/* allocate all of the leftover color cells */
#ifdef _NO_PROTO
void updateColormap()
#else
void updateColormap(void)
#endif
{
Pixel pix;
Pixel freePixels[256];
int i, numFree;
XColor c[256], cell;
char buf[256];
FILE *file;
    if (appData.colorfile)
    {
	if (file = fopen(appData.colorfile, "r"))
	{
	    while (XAllocColorCells(display, cmap, False, NULL, 0, &pix, 1))
	    {
		if (pix & 0xf)
		{
		    cell.red = cell.green = cell.blue = 0;
		    cell.flags = DoRed | DoGreen | DoBlue;
		    cell.pixel = pix;
		    cols.changable[pix] = True;
		    cols.freeCell[pix] = False;
		    XStoreColor(display, cmap, &cell);
		}
		else
		{
		    XFreeColors(display, cmap, &pix, 1, 0);
		    break;
		}
	    }
	    while (fgets(buf, 256, file))
	    {
		buf[strlen(buf)-1] = 0;		/* remove newline char */
		if (!XParseColor(display, cmap, buf, &cell))
		    continue;
		if  (XAllocColorCells(display, cmap, False, NULL, 0, &pix, 1))
		{
		    cols.changable[pix] = True;
		    cols.freeCell[pix] = False;
		    cell.pixel = pix;
		    XStoreColor(display, cmap, &cell);
		}
		else
		    break;
	    }
	}
	fclose(file);
    }
    while  (XAllocColorCells(display, cmap, False, NULL, 0, &pix, 1))
    {
	cols.changable[pix] = False;
	cols.freeCell[pix] = True;
    }
    
    for (i=0; i<256; i++)
	c[i].pixel = i;
    XQueryColors(display, cmap, c, 256);
    numFree = 0;
    for (i=0; i<256; i++)
    {
	cols.cells[i].red   = c[i].red & 0xff00;
	cols.cells[i].green = c[i].green & 0xff00;
	cols.cells[i].blue  = c[i].blue & 0xff00;
	if (cols.freeCell[i])
	    freePixels[numFree++] = i;
    }
    if (numFree)
	XFreeColors(display, cmap, freePixels, numFree, 0);
}

/* remember the symbolic name of a color */
#ifdef _NO_PROTO
void setSymbolicName(index, name)
int index; String name;
#else
void setSymbolicName(int index, String name)
#endif
{
    if (index < 0 || index >= 256)
	return;
    if (symbolicName[index])
	XtFree(symbolicName[index]);
    if (name)
	symbolicName[index] = XtNewString(name);
    else
	symbolicName[index] = NULL;
}

/* get the symbolic name of a color */
#ifdef _NO_PROTO
String getSymbolicName(index)
int index;
#else
String getSymbolicName(int index)
#endif
{
    if (index < 0 || index >= 256)
	return(NULL);
    else
	return(symbolicName[index]);
}

/* update our knowlege of colors - XReadPixmap may allocate some colors */
#ifdef _NO_PROTO
void newLoadedPixels(pixels, npixels, nsyms, colorTable, symPixels)
unsigned long *pixels; int npixels;
    int nsyms; char ***colorTable; unsigned long *symPixels;
#else
void newLoadedPixels(unsigned long *pixels, int npixels,
    int nsyms, char ***colorTable, unsigned long *symPixels)
#endif
{
unsigned long pix;
int i;
XColor c;
    for (i=0; i<npixels; i++)
    {
	pix = pixels[i];
	if (cols.freeCell[pix])
	{
	    c.pixel = pix;
	    XQueryColor(display, cmap, &c);
	    cols.cells[pix].red   = c.red & 0xff00;
	    cols.cells[pix].green = c.green & 0xff00;
	    cols.cells[pix].blue  = c.blue & 0xff00;
	}
	cols.freeCell[pix] = False;
	cols.changable[pix] = False;
    }
    for (i=0; i<nsyms; i++)
    {
	if (colorTable[i][1])
	{
	    setSymbolicName(symPixels[i], colorTable[i][1]);
	}
    }
    setCurrentPixel(curPixel);
}

/* update our knowlege of colors - XmCreateXXX may allocate some colors */
#ifdef _NO_PROTO
void newAllocatedPixels()
#else
void newAllocatedPixels()
#endif
{
int i;
XColor cells[256];

    /* query all colors and check to see if any previously free cells */
    /* are no longer black */

    for (i=0; i<256; i++)
	cells[i].pixel = i;
    XQueryColors(display, cmap, cells, 256);
    for (i=0; i<256; i++)
    {
	if (cols.freeCell[i] && 
		(cells[i].red || cells[i].green || cells[i].blue))
	{
	    cols.freeCell[i] = False;
	    cols.changable[i] = False;
	    cols.cells[i].red   = cells[i].red & 0xff00;
	    cols.cells[i].green = cells[i].green & 0xff00;
	    cols.cells[i].blue  = cells[i].blue & 0xff00;
	}
    }
}
