#include "declare.h"
#include "Argtricks.h"
#include <X11/Wc/WcCreate.h>
#include <Xm/TextF.h>

Widget redScale, greenScale, blueScale, colorPallette, colorText, 
	colorBlock, colorList, symName;

#define MAX_COLORS	1024

XmString xmsList[MAX_COLORS];
short redValue[MAX_COLORS], greenValue[MAX_COLORS], blueValue[MAX_COLORS];
String sList[MAX_COLORS];
Pixel fgPixel, bgPixel;
int numColors;
Dimension pWidth, pHeight, pShadow;
GC colgc;

#ifdef _NO_PROTO
String getSymbolicName();
#else
String getSymbolicName(int);
#endif

#ifdef _NO_PROTO
void redrawWidget(w)
Widget w;
#else
void redrawWidget(Widget w)
#endif
{
    if (XtIsRealized(w))
	XClearArea(XtDisplayOfObject(w), XtWindowOfObject(w),
	    0, 0, 0, 0, True);
}

#ifdef _NO_PROTO
String getRGBText(red, green, blue)
int red; int green; int blue;
#else
String getRGBText(int red, int green, int blue)
#endif
{
int i;
static char buf[12];
    /* look for color name match */
    for (i=0; i<numColors; i++)
    {
	if (red == redValue[i] && green == greenValue[i] 
		&& blue == blueValue[i])
	    return(sList[i]);
    }
    sprintf(buf, "#%02x%02x%02x", red, green, blue);
    return(buf);
}

#ifdef _NO_PROTO
void lookupColorName(red, green, blue)
int red; int green; int blue;
#else
void lookupColorName(int red, int green, int blue)
#endif
{
    XmTextFieldSetString(colorText, getRGBText(red, green, blue));
}
    

#ifdef _NO_PROTO
void setScaleRGB(red, green, blue, changable)
int red; int green; int blue; int changable;
#else
void setScaleRGB(int red, int green, int blue, int changable)
#endif
{
DeclareArgs(2);

    red >>=8;
    green >>=8;
    blue >>=8;

    FirstArg(XmNvalue, red);
    XtSetValues(redScale, Args, ArgCount);

    FirstArg(XmNvalue, green);
    XtSetValues(greenScale, Args, ArgCount);

    FirstArg(XmNvalue, blue);
    XtSetValues(blueScale, Args, ArgCount);

    XtSetSensitive(redScale, changable);
    XtSetSensitive(greenScale, changable);
    XtSetSensitive(blueScale, changable);

    lookupColorName(red, green, blue);
}

/* ARGSUSED */
#ifdef _NO_PROTO
static void drawPalletteCB(wid, unused, unused_too)
Widget wid; XtPointer unused; XtPointer unused_too;
#else
static void drawPalletteCB(Widget wid, XtPointer unused, XtPointer unused_too)
#endif
{
    int i, w, h, x, y, dx, dy;
    Window win = XtWindow(colorPallette);
    int red, green, blue, changable, val;

    w = pWidth - (pShadow*2+2);
    h = pHeight - (pShadow*2+2);
    dx = w / 16;
    dy = h / 16;

    for (i=0; i<256; i++)
    {
	x = i % 16;
	y = i / 16;
	x *= dx;
	y *= dy;
	x += pShadow + 2;
	y += pShadow + 2;
	XSetForeground(display, colgc, i);
	XFillRectangle(display, win, colgc, x, y, dx-2, dy-2);
	getRGB(i, &red, &green, &blue, &changable);
	if (!changable)
	{
	    /* compute intensity */
	    val = (red * 30) + (green * 59) + (blue * 11);
	    val >>= 16;
	    if (val < 70)
		XSetForeground(display, colgc, fgPixel);
	    else
		XSetForeground(display, colgc, bgPixel);
	    XDrawLine(display, win, colgc, x, y, x + dx - 3, y + dy - 3);
	    XDrawLine(display, win, colgc, x + dx - 3, y, x, y + dy - 3);
	}
    }
    x = (int)(curPixel % 16) * dx;
    y = (int)(curPixel / 16) * dy;
    XSetForeground(display, colgc, fgPixel);
    XDrawRectangle(display, win, colgc, x+pShadow, y+pShadow, dx+1, dy+1);
    XDrawRectangle(display, win, colgc, x+pShadow+1, y+pShadow+1, dx-1, dy-1);
}

#ifdef _NO_PROTO
void setCurrentPixel(p)
int p;
#else
void setCurrentPixel(int p)
#endif
{
    int red, green, blue, changable;
    Window win = XtWindow(colorPallette);
    DeclareArgs(2);
    int x, y, dx, dy, c;
    String sym;

    if (XtIsRealized(colorPallette))
    {
	dx = (pWidth - (pShadow*2+2)) / 16;
	dy = (pHeight - (pShadow*2+2)) / 16;

	x = (int)(curPixel % 16) * dx;
	y = (int)(curPixel / 16) * dy;
	XSetForeground(display, colgc, bgPixel);
	XDrawRectangle(display, win, colgc, 
		x+pShadow, y+pShadow, dx+1, dy+1);
	XDrawRectangle(display, win, colgc,
		x+pShadow+1, y+pShadow+1, dx-1, dy-1);

	curPixel = p;
	x = (int)(curPixel % 16) * dx;
	y = (int)(curPixel / 16) * dy;
	XSetForeground(display, colgc, fgPixel);
	XDrawRectangle(display, win, colgc,
		x+pShadow, y+pShadow, dx+1, dy+1);
	XDrawRectangle(display, win, colgc,
		x+pShadow+1, y+pShadow+1, dx-1, dy-1);
    }
    else
	curPixel = p;

    FirstArg(XmNbackground, curPixel);
    XtSetValues(colorBlock, Args, ArgCount);

    getRGB((int)curPixel, &red, &green, &blue, &changable);
    setScaleRGB(red, green, blue, changable);
    sym = getSymbolicName((int)curPixel);
    if (sym)
	XmTextFieldSetString(symName, sym);
    else
	XmTextFieldSetString(symName, "");
    XmTextFieldShowPosition(symName, 0);
}

/* ARGSUSED */
#ifdef _NO_PROTO
static void inputPalletteCB(wid, unused, cb)
Widget wid; XtPointer unused; XmAnyCallbackStruct *cb;
#else
static void inputPalletteCB(Widget wid, XtPointer unused, 
	XmAnyCallbackStruct *cb)
#endif
{
    int x, y, dx, dy, p;
    XButtonEvent *e = &(cb->event->xbutton);

    x = e->x;
    y = e->y;
    x -= pShadow;
    y -= pShadow;
    dx = (pWidth - (pShadow*2+2)) / 16;
    dy = (pHeight - (pShadow*2+2)) / 16;
    x /= dx;
    y /= dy;

    p = y * 16 + x;
    if (p < 0 || p > 255)
	return;
    setCurrentPixel(p);
}

#ifdef _NO_PROTO
void getScaleRGB(red, green, blue)
int *red; int *green; int *blue;
#else
void getScaleRGB(int *red, int *green, int *blue)
#endif
{
    XmScaleGetValue(redScale, red);
    XmScaleGetValue(greenScale, green);
    XmScaleGetValue(blueScale, blue);
    lookupColorName(*red, *green, *blue);
    *red <<= 8;
    *green <<= 8;
    *blue <<= 8;
}

/* ARGSUSED */
#ifdef _NO_PROTO
static void changeScaleCB(wid, unused, unused_too)
Widget wid; XtPointer unused; XtPointer unused_too;
#else
static void changeScaleCB(Widget wid, XtPointer unused, XtPointer unused_too)
#endif
{
int red, green, blue;
    getScaleRGB(&red, &green, &blue);
    setRGB((int)curPixel, red, green, blue);
}

/* ARGSUSED */
#ifdef _NO_PROTO
static void newColorCB(wid, unused, unused_too)
Widget wid; XtPointer unused; XtPointer unused_too;
#else
static void newColorCB(Widget wid, XtPointer unused, XtPointer unused_too)
#endif
{
    int pix;
    int red, green, blue, changable;

    newColor(&pix, XmTextFieldGetString(colorText));
    if (pix >= 0)
	setCurrentPixel(pix);
    else
    {
	popupMessage(XPE_Warning, "no more available pixels!",
		NULL, NULL, NULL);
	return;
    }

    getRGB((int)curPixel, &red, &green, &blue, &changable);
    setScaleRGB(red, green, blue, changable);
    redrawWidget(colorPallette);
}

#ifdef _NO_PROTO
void setColorName(pos, set)
int pos; int set;
#else
void setColorName(int pos, int set)
#endif
{
int red, green, blue, changable;

    XmTextFieldSetString(colorText, sList[pos]);
    if (set)
    {
	setRGB((int)curPixel, 
		redValue[pos] << 8,
		greenValue[pos] << 8,
		blueValue[pos] << 8);

	/* get value because it may not be allowed to change */
	getRGB((int)curPixel, &red, &green, &blue, &changable);
	setScaleRGB(red, green, blue, changable);
    }
}

/* ARGSUSED */
#ifdef _NO_PROTO
void symNameCB(w, unused, unused_too)
Widget w; XtPointer unused; XtPointer unused_too;
#else
void symNameCB(Widget w, XtPointer unused, XtPointer unused_too)
#endif
{
    String s;
    s = XmTextFieldGetString(symName);
    if (strlen(s) > 0)
	setSymbolicName((int)curPixel, s);
    else
	setSymbolicName((int)curPixel, NULL);
    XtFree(s);
}

/* ARGSUSED */
#ifdef _NO_PROTO
void textSelectCB(w, unused, unused_too)
Widget w; XtPointer unused; XtPointer unused_too;
#else
void textSelectCB(Widget w, XtPointer unused, XtPointer unused_too)
#endif
{
    String s;
    XColor cell;
    char buf[256];
    int red, green, blue, changable;

    s = XmTextFieldGetString(w);
    if (!XParseColor(display, cmap, s, &cell))
    {
	sprintf(buf, "Invalid color name: '%s'", s);
	popupMessage(XPE_Warning, buf, NULL, NULL, NULL);
	return;
    }
    cell.red &=
    cell.red   &= 0xff00;
    cell.green &= 0xff00;
    cell.blue  &= 0xff00;

    setRGB((int)curPixel, cell.red, cell.green, cell.blue);
    getRGB((int)curPixel, &red, &green, &blue, &changable);
    setScaleRGB(red, green, blue, changable);
}

/* ARGSUSED */
#ifdef _NO_PROTO
void listSelectCB(w, unused, cb)
Widget w; XtPointer unused; XmListCallbackStruct *cb;
#else
void listSelectCB(Widget w, XtPointer unused,
    XmListCallbackStruct *cb)
#endif
{
    setColorName(cb->item_position-1, 
	cb->reason == XmCR_DEFAULT_ACTION);
}

#ifdef _NO_PROTO
initColorWindow()
#else
initColorWindow()
#endif
{
    WcRegisterCallback(appCon, "drawPalletteCB", 
	(XtCallbackProc)drawPalletteCB, NULL);
    WcRegisterCallback(appCon, "inputPalletteCB", 
	(XtCallbackProc)inputPalletteCB, NULL);
    WcRegisterCallback(appCon, "listSelectCB", 
	(XtCallbackProc)listSelectCB, NULL);
    WcRegisterCallback(appCon, "newColorCB", 
	(XtCallbackProc)newColorCB, NULL);
    WcRegisterCallback(appCon, "changeScaleCB", 
	(XtCallbackProc)changeScaleCB, NULL);
}

#ifdef _NO_PROTO
createColorWindow(parent)
Widget parent;
#else
createColorWindow(Widget parent)
#endif
{
    DeclareArgs(10);
    Widget colorBoard;
    Widget currentColor;
    FILE *file;
    int red, green, blue, changable;
    String sym;

    WcWidgetCreation(parent);

    curPixel = 0;

    if (file = fopen(RGB_FNAME, "r"))
    {
	int r, g, b, n;
	char name[256];
	XmString xs;
	String s;

	numColors = 0;
	while (numColors < MAX_COLORS)
	{
	    n = fscanf(file, "%d %d %d %[^\n]\n", &r, &g, &b, name);
	    if (n != 4)
		break;
	    s = XtNewString(name);
	    /* strcpy(s, name); */
	    xs = XS(s);
	    xmsList[numColors] = xs;
	    sList[numColors] = s;
	    redValue[numColors] = r;
	    greenValue[numColors] = g;
	    blueValue[numColors] = b;
	    numColors++;
	}
	fclose(file);
    }
    else
    {
	printf("cannot open color database '%s'\n", RGB_FNAME);
	exit(-1);
    }

    redScale = WcFullNameToWidget(parent, "*cbRedScale");
    greenScale = WcFullNameToWidget(parent, "*cbGreenScale");
    blueScale = WcFullNameToWidget(parent, "*cbBlueScale");
    colorPallette = WcFullNameToWidget(parent, "*cbPallette");
    colorText = WcFullNameToWidget(parent, "*cbText");
    colorBlock = WcFullNameToWidget(parent, "*cbBlock");
    colorList = WcFullNameToWidget(parent, "*cbList");
    symName = WcFullNameToWidget(parent, "*cbSymName");

    FirstArg(XmNitemCount, numColors);
    NextArg(XmNitems, xmsList);
    XtSetValues(colorList, Args, ArgCount);

    FirstArg(XmNwidth, &pWidth);
    NextArg(XmNheight, &pHeight);
    NextArg(XmNshadowThickness, &pShadow);
    NextArg(XmNforeground, &fgPixel);
    NextArg(XmNbackground, &bgPixel);
    XtGetValues(colorPallette, Args, ArgCount);

    colgc = XCreateGC(display,
	RootWindowOfScreen(XtScreenOfObject(parent)), 0, NULL);

    setCurrentPixel(curPixel);
}
