#include "declare.h"
#include <Xm/Xm.h>
#include <Xm/MessageB.h>
#include "Argtricks.h"

#ifdef _NO_PROTO
void callPopDownDestroy(w, popw, reason)
Widget w; Widget popw; XmAnyCallbackStruct *reason;
#else
void callPopDownDestroy(Widget w, Widget popw, XmAnyCallbackStruct *reason)
#endif
{
    XtUnmanageChild(popw);
    XtDestroyWidget(popw);
}

/*
 * display a message and force the user to click to continue
 */
#ifdef _NO_PROTO
void popupMessage(type, message, okCallback, cancelCallback, helpCallback)
	int type;
	char *message; 
	XtCallbackProc okCallback; 
	XtCallbackProc cancelCallback; 
	XtCallbackProc helpCallback;
#else
void popupMessage(int type, char *message, 
	XtCallbackProc okCallback, 
	XtCallbackProc cancelCallback, 
	XtCallbackProc helpCallback)
#endif
{
DeclareArgs(5);
Widget msgW;
XmString msg;
String title;
XrmValue from, to;

    /* be sure that we are visible */
    if (XtIsRealized(topLevel))
	XtMapWidget(topLevel);

    FirstArg(XmNautoUnmanage, True);

    if (type == XPE_Error)
    {
	NextArg(XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
	title = "XPE Error";
	msgW = XmCreateErrorDialog(topLevel, "xpeError",
	    Args, ArgCount);
    }
    else if (type == XPE_Help)
    {
	title = "XPE Help";
	msgW = XmCreateInformationDialog(topLevel, "xpeHelp",
	    Args, ArgCount);
    }
    else if (type == XPE_Info)
    {
	title = "XPE Info";
	msgW = XmCreateInformationDialog(topLevel, "xpeInfo",
	    Args, ArgCount);
    }
    else if (type == XPE_Warning)
    {
	title = "XPE Warning";
	NextArg(XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
	msgW = XmCreateWarningDialog(topLevel, "xpeWarn",
	    Args, ArgCount);
    }
    else
    {
	title = "XPE Message";
	msgW = XmCreateMessageDialog(topLevel, "xpeMsg",
	    Args, ArgCount);
    }

    /* Use the built in string to font converter since it will */
    /* convert \n into newlines - and XmStringCreateSimple will not */
    /* The XmString will be freed when the refererence widget (msgW) */
    /* is freed. */

    from.addr = (XtPointer)message;
    from.size = strlen(message) + 1;
    to.addr = (XtPointer)&msg;
    to.size = sizeof(XmString);

    if (!XtConvertAndStore(msgW, XmRString, &from, XmRXmString, &to))
    {
	fprintf(stderr, "XPE: Cannot convert message into compound string!\n");
	fprintf(stderr, "XPE: Fatal error!\n");
	exit(-1);
    }

    /* now set the message */
    FirstArg(XmNmessageString, msg);
    XtSetValues(msgW, Args, ArgCount);

    /* which buttons? */
    if (cancelCallback)
	XtAddCallback(msgW, XmNcancelCallback, cancelCallback, 0);
    else
	XtUnmanageChild(XmMessageBoxGetChild(msgW, XmDIALOG_CANCEL_BUTTON));

    if (helpCallback)
	XtAddCallback(msgW, XmNhelpCallback, helpCallback, 0);
    else
	XtUnmanageChild(XmMessageBoxGetChild(msgW, XmDIALOG_HELP_BUTTON));

    if (okCallback)
	XtAddCallback(msgW, XmNokCallback, okCallback, 0);

    /* ok also pops down this message */
    XtAddCallback(msgW, XmNokCallback, 
		(XtCallbackProc)callPopDownDestroy, (XtPointer)msgW);

    /* set window title */
    FirstArg(XmNtitle, title);
    XtSetValues(XtParent(msgW), Args, ArgCount);

    msg = XS(title);
    FirstArg(XmNdialogTitle, msg);
    XtSetValues(msgW, Args, ArgCount);
    XmStringFree(msg);

    /* update color map info */
    newAllocatedPixels();

    /* display it */
    XtManageChild(msgW);

    /* let's get things updated */
    XmUpdateDisplay(topLevel);
}
