/*
 * utility functions:
 *
 *	askFile - put up File Selection box and call user func if ok
 *
 *	askSize - put up a dialog for widget size/scale
 */
#include "declare.h"
#include "Argtricks.h"
#include <X11/Wc/WcCreate.h>
#include <X11/cursorfont.h>
#include <X11/StringDefs.h>

#include <Xm/FileSB.h>
#include <Xm/BulletinB.h>
#include <Xm/MessageB.h>
#include <Xm/Scale.h>
#include <Xm/TextF.h>

/***********************************************************
 * askFile
 **********************************************************/

Widget fileSB = NULL;

typedef void (*OkFunction)();

OkFunction fsDoIt = 0;
XtPointer fsData = 0;

#ifdef _NO_PROTO
void doFSOK(w, data, cb)
Widget w; XtPointer data; XmFileSelectionBoxCallbackStruct *cb;
#else
void doFSOK(Widget w, XtPointer data, XmFileSelectionBoxCallbackStruct *cb)
#endif
{
    String file;

    XtUnmanageChild(fileSB);
    XmUpdateDisplay(topLevel);

    XmStringGetLtoR(cb->value, XmSTRING_DEFAULT_CHARSET, &file);
    (*fsDoIt)(fsData, file);
    XtFree(file);
}

#ifdef _NO_PROTO
void askFile(doit, title, data, mask)
OkFunction doit; String title; XtPointer data; String mask;
#else
void askFile(OkFunction doit, String title, XtPointer data, String mask)
#endif
{
DeclareArgs(2);
XmString xs;

    fsDoIt = doit;
    fsData = data;
    if (!fileSB)
    {
	FirstArg(XmNautoUnmanage, True);
	NextArg(XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
	fileSB = XmCreateFileSelectionDialog(topLevel, "fileSB", 
		Args, ArgCount);
	XtAddCallback(fileSB, XmNokCallback, (XtCallbackProc)doFSOK, NULL);
    }
    FirstArg(XmNtitle, title);
    XtSetValues(XtParent(fileSB), Args, ArgCount);

    xs = XmStringCreateSimple(mask);
    XmFileSelectionDoSearch(fileSB, xs);
    XmStringFree(xs);
    
    /* be sure that we are visible */
    if (XtIsRealized(topLevel))
	XtMapWidget(topLevel);

    XtManageChild(fileSB);

    /* update color map info */
    newAllocatedPixels();

    XmUpdateDisplay(topLevel);
}

/***********************************************************
 * askSize
 **********************************************************/

Widget sizeDialog = NULL, sdWidth, sdHeight, sdScale;

OkFunction sizeDoIt = 0;
XtPointer sizeData = 0;

#ifdef _NO_PROTO
void doSizeOK(w, data, unused_too)
Widget w; XtPointer data; XtPointer unused_too;
#else
void doSizeOK(Widget w, XtPointer data, XtPointer unused_too)
#endif
{
    int width, height, scale;
    String s;

    s = XmTextFieldGetString(sdWidth);
    width = atoi(s);
    XtFree(s);

    s = XmTextFieldGetString(sdHeight);
    height = atoi(s);
    XtFree(s);

    XmScaleGetValue(sdScale, &scale);

    (*sizeDoIt)(sizeData, width, height, scale);
}

#ifdef _NO_PROTO
void askSize(doit, title, data, width, height, scale)
OkFunction doit; String title; XtPointer data; 
	int width; int height; int scale;
#else
void askSize(OkFunction doit, String title, XtPointer data, 
	int width, int height, int scale)
#endif
{
DeclareArgs(2);
char buf[16];

    sizeDoIt = doit;
    sizeData = data;

    if (!sizeDialog)
    {
	Widget but;

	FirstArg(XmNautoUnmanage, True);
	NextArg(XmNdialogStyle, XmDIALOG_FULL_APPLICATION_MODAL);
	sizeDialog = XmCreateBulletinBoardDialog(topLevel, "sizeDialog", 
		Args, ArgCount);

	WcWidgetCreation(sizeDialog);
	sdWidth = WcFullNameToWidget(sizeDialog, "*sdWidth");
	sdHeight = WcFullNameToWidget(sizeDialog, "*sdHeight");
	sdScale = WcFullNameToWidget(sizeDialog, "*sdScale");
	but = WcFullNameToWidget(sizeDialog, "*sdApply");

	XtAddCallback(but, XmNactivateCallback, 
		(XtCallbackProc)doSizeOK, NULL);
    }
    FirstArg(XmNtitle, title);
    XtSetValues(XtParent(sizeDialog), Args, ArgCount);

    sprintf(buf, "%d", width);
    XmTextFieldSetString(sdWidth, buf);
    sprintf(buf, "%d", height);
    XmTextFieldSetString(sdHeight, buf);
    XmScaleSetValue(sdScale, scale);

    /* be sure that we are visible */
    if (XtIsRealized(topLevel))
	XtMapWidget(topLevel);

    XtManageChild(sizeDialog);

    /* update color map info */
    newAllocatedPixels();

    XmUpdateDisplay(topLevel);
}

/***********************************************************
 * contextHelp
 **********************************************************/

typedef struct {
    String helpMsg;
} HelpData, *HelpDataPtr;

XtResource helpResources[] = {
    {"helpMsg", "HelpMsg", XmRString, sizeof(String),
	XtOffsetOf(HelpData, helpMsg), XtRImmediate, NULL},
};

/* ARGSUSED */
#ifdef _NO_PROTO
void helpMsgCB(wid, type, unused_too)
Widget wid; String type; XtPointer unused_too;
#else
void helpMsgCB(Widget wid, String type, XtPointer unused_too)
#endif
{
    static Cursor helpCursor = NULL;
    HelpData helpData;
    XtCallbackStatus stat;

    if (!helpCursor)
	helpCursor = XCreateFontCursor(display, XC_question_arrow);
    
    if (strcmp(type, "locate") == 0)
    {
	wid = XmTrackingLocate(topLevel, helpCursor, False);
	if (!wid)
	    wid = topLevel;
	
	/* fall thru for top level */
	while (wid != topLevel)
	{
	    /* search for Motif callback list */
	    stat =  XtHasCallbacks(wid, XmNhelpCallback);

	    /* fall out if not a Motif widget */
	    if (stat == XtCallbackNoList)
		break;
	    /* else check parent if no help for this one */
	    else if (stat == XtCallbackHasNone)
		wid = XtParent(wid);
	    /* else call it! */
	    else
	    {
		XtCallCallbacks(wid, XmNhelpCallback, (XtPointer)"me");
		return;
	    }
	}
    }

    XtGetApplicationResources(wid, &helpData,
	helpResources, XtNumber(helpResources), NULL, 0);

    if (helpData.helpMsg)
    {
	popupMessage(XPE_Help, helpData.helpMsg, NULL, NULL, NULL);

	/* update color map info */
	newAllocatedPixels();
    }
    else
    {
	popupMessage(XPE_Error, "Cannot find help information",
		NULL, NULL, NULL);

	/* update color map info */
	newAllocatedPixels();
    }
}
